
# installib.py
#
#    ========== licence begin LGPL
#    Copyright (C) 2000 SAP AG
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#    ========== licence end
#


import sys
import string
import re
import os
import tempfile

import repman

_version = 0.2

_okExit = 0
_repmanErrorExit = 3
_pythonErrorExit = 4

#
#
# common code used by all installation modules
#

verbose = None

def repmanCmd (session, cmdStr):
    #if verbose:
    #    print cmdStr,
    try:
        result = session.cmd (cmdStr)
        #if verbose:
        #    print '=> OK', result
    except repman.RepManServError, err:
        #if verbose:
        #    print '=> repman failed', err
        raise repman.RepManServError, err, sys.exc_traceback
    except:
        #print '=> unexpected exception'
        kind, val, traceback = sys.exc_info()
        raise kind, val, traceback

def sqlOK (session, cmd):
    try:
        repmanCmd (session, cmd)
        ok = 1
    except repman.RepManServError:
        ok = None
    return ok

def stopIfSqlNotOK (session, cmd):
    try:
        repmanCmd (session, cmd)
    except repman.RepManServError, err:
        if (err.sqlCode == 0) or (err.sqlCode == None):
            raise err  # this is a ReplMan error
        stopInstall (session, err.sqlCode)

def sqlRC (session, cmd):
    try:
        repmanCmd (session, cmd)
        rc = 0
    except repman.RepManServError, err:
        if (err.sqlCode == 0) or (err.sqlCode == None):
#           print 'no match'
            raise err  # this is a ReplMan error
        rc = err.sqlCode
    return rc

def stopInstall (session, err):
    repmanCmd (session, 'INSTALLATION OFF')
    fatalError ('unexpected Error ' + `err`)

def stopIfSqlNotErr (session, expectErr, cmd):
    rc = sqlRC (session, cmd)
    if not (rc in [0, expectErr]):
        stopInstall(session, rc)


def createReplaceView (session, viewname, cmdtail):

    rc = sqlRC (session, 'CREATE OR REPLACE VIEW ' + viewname + ' ' + cmdtail)

    if rc == -1026:
        stopIfSqlNotErr (session, -4004, 'DROP VIEW ' + viewname)

        stopIfSqlNotOK (session, 'CREATE VIEW ' + viewname + ' ' + cmdtail)
    else:
        if rc != 0:
            stopInstall(session, rc)


def existsTable (session, tableName):
    return sqlOK (session, "EXISTS TABLE %s" % tableName)

def dropView (session, viewName):
    if sqlOK (session, "SHOW VIEW %s" % viewName):
        stopIfSqlNotOK (session, "DROP VIEW %s" % viewName)

def dropTable (session, tableName):
    if existsTable (session, tableName):
        stopIfSqlNotOK (session, "DROP TABLE %s" % tableName)

def include (session, options, *files):
    for file in files:
        install = __import__ (file).install
        if options.verbose:
            doc = install.__doc__
            if not doc:
                doc = file
            print doc
        install (session, options)

def requireSysdba (session):
    sqlOK (session, "SHOW (thisuser) USER CURRENT")
    sqlOK (session, "SHOW (sysdba_dom) USER LOCALSYSDBA")

    if not sqlOK (session, """SELECT thisuser.username from thisuser, sysdba_dom
            where thisuser.username = sysdba_dom.username"""):
        stopInstall (session, "User is not SYSDBA")

def dataLoad (session, loadcmd, data = None):
    if data != None:
        # write data to tmpfile
        tmpname = tempfile.mktemp ()
        stream = open (tmpname, 'w')
        stream.write (data)
        stream.close ()
        # patch load cmd
        match = re.search ('INFILE +[*]', loadcmd)
        if match != None:
            start, end = match.span ()
            loadcmd = loadcmd [:start] + ("INFILE '%s'" % tmpname) + loadcmd [end:]
    # execute command
    repmanCmd (session, loadcmd)
    # clean up
    if data != None:
        os.unlink (tmpname)

def switchToDefaultUser (session, options):
    pos = string.find (options.user, ',')
    username = options.user [:pos]
    pwd = options.user [pos + 1:]
    session.cmd ('use user %s %s' % (username, pwd))

def fatalError (msg):
    sys.stderr.write (msg + '\n')
    sys.exit (2)

def getFileForTraceback (tb):
    fname = tb.tb_frame.f_code.co_filename
    return fname

def tracebackSize (tb):
    if tb == None:
        return 0
    nesting = tracebackSize (tb.tb_next)
    if nesting == 0:
        fname = getFileForTraceback (tb)
        if string.lower (os.path.basename (fname)) == 'installib.py':
            return 0
    return nesting + 1

def dumpInstallError (err, logfile):
    import traceback
    write = logfile.write
    exctype, excvalue, tb = sys.exc_info ()
    nesting = tracebackSize (tb)
    write ('---------Traceback---------------\n')
    traceback.print_tb (tb, nesting, logfile)
    if hasattr (err, 'cmd') and err.cmd:
        write ('---------Command-----------------\n')
        write (err.cmd + '\n')
    write ( '---------Error-------------------\n')
    write (str (err) + '\n')

def dumpSyntaxError (logfile):
    import traceback
    write = logfile.write
    exctype, excvalue, tb = sys.exc_info ()
    write ('---------Traceback---------------\n')
    #traceback.print_tb (tb, None, logfile)
    traceback.print_exception (exctype, excvalue, tb, None, logfile)
    write ( '---------Error-------------------\n')
    write ("%s: %s\n" % (exctype, excvalue))
    lines = traceback.format_exception_only(exctype, excvalue)

def dumpPythonError (logfile):
    import traceback
    write = logfile.write
    exctype, excvalue, tb = sys.exc_info ()
    write ('---------Traceback---------------\n')
    traceback.print_tb (tb, None, logfile)
    write ( '---------Error-------------------\n')
    write ("%s: %s\n" % (exctype, excvalue))

def locateModule (moduleName):
    import imp
    import os
    stream, fname, dummy = imp.find_module (moduleName)
    result = os.path.dirname (fname)
    if result == '':
        result = '.'
    return result

def connect (options):
    if not options.user:
        fatalError ('user name required (-u <user>,<pwd>)')
    if not options.db:
        fatalError ('database name required (-d <dbname>)')
    pos = string.find (options.user, ',')
    if pos == -1:
        fatalError ('wrong format, use -u <user>,<pwd>')
    username = options.user [:pos]
    pwd = options.user [pos + 1:]
    if options.remote:
        node = options.node
    else:
        node = ''
    session = repman.RepMan (node, options.db)
    if options.node == '':
        session.cmd ('use user %s %s' % (username, pwd))
    else:
        session.cmd ('use user %s %s serverdb %s on %s' % (username, pwd, options.db, options.node ))
    return session

class StdoutLog:
    def write (self, text):
        sys.stdout.write (text)

class TeeLog:
    def __init__ (self, fname):
        self.logfile = open (fname, 'w')

    def write (self, text):
        self.logfile.write (text)
        sys.stdout.write (text)

_dblngMapping = {
    'en': 'ENG',
    'de': 'DEU',
}

def connectAndInstall (installRoutine, docString = None, args = None):
    if docString == None:
        docSting = installRoutine.__doc__
    import optlib
    options, args = optlib.parseArgs (_options (), docString, args)
    global verbose
    verbose = options.verbose
    session = connect (options)
    options.installdir = locateModule('lsystab')
    options.dblng = _dblngMapping [options.lng]
    if options.errfile:
        options.errLog = TeeLog (options.errfile)
    else:
        options.errLog = StdoutLog ()
    try:
        installRoutine (session, options)
        exitCode = _okExit
    except repman.RepManServError, err:
        dumpInstallError (err, options.errLog)
        exitCode = _repmanErrorExit
    except SyntaxError:
        dumpSyntaxError (options.errLog)
        exitCode = _pythonErrorExit
    except:
        dumpPythonError (options.errLog)
        exitCode = _pythonErrorExit
    session = None
    sys.exit (exitCode)

def main (options, args):
    session = connect (options)
    install (session)
    session.release ()

def _options ():
    return [
        # (optstring, varname, typechar, default, help)
	('u', 'user', ':', None, 'user name [<user>,<pwd>]'),
	('dompwd', 'dompwd', ':', None, 'domain password'),
	('d', 'db', ':', None, 'db name'),
        ('n', 'node', ':', '', 'server name'),
        ('r', 'remote',':', None, 'connect to remote RepMan server'),
        ('l', 'lng', ':', 'en', 'language: de, en ...'),
        ('fdir', 'fdir', ':', '.', 'inputfile directory'),
        ('v', 'verbose', '', None, 'verbose output'),
        ('err', 'errfile', ':', None, 'name of error log'),
        ('version', None, '', _version, 'version information'),
        ]

if __name__ == '__main__':
    import optlib
    optlib.optMain2 (main, _options())


